@@ -9,26 +9,23 @@
 #' @export
 rtruncgamma <- rtrunc.gamma <- function(n, shape, rate = 1, scale = 1 / rate,
                                         a = 0, b = Inf) {
+  if (!missing(rate) && !missing(scale)) {
+    stop("specify 'rate' or 'scale' but not both")
+  }
   class(n) <- "trunc_gamma"
   sampleFromTruncated(mget(ls()))
 }
 
 #' @export
-dtrunc.trunc_gamma <- function(y, eta, a = 0, b = Inf) {
-  parm <- natural2parameters.trunc_gamma(eta)
-  dens <- ifelse((y < a) | (y > b), 0, dgamma(y, shape = parm[1], rate = parm[2]))
-  if (!missing(a)) {
-    F.a <- pgamma(a, shape = parm[1], rate = parm[2])
-  } else {
-    F.a <- 0
+dtrunc.trunc_gamma <- function(
+  y, shape, rate = 1, scale = 1 / rate, eta, a = 0, b = Inf, ...
+) {
+  if (missing(eta)) {
+    eta <- parameters2natural.parms_gamma(c("shape" = shape, "rate" = rate, "scale" = scale))
   }
-  if (!missing(b)) {
-    F.b <- pgamma(b, shape = parm[1], rate = parm[2])
-  } else {
-    F.b <- 1
-  }
-  const <- 1 / (F.b - F.a)
-  return(dens * const)
+  parm <- natural2parameters.parms_gamma(eta)
+  dens <- rescaledDensities(y, a, b, dgamma, pgamma, parm["shape"], parm["rate"])
+  return(dens)
 }
 
 #' @rdname dtrunc
@@ -36,13 +33,13 @@ dtrunc.trunc_gamma <- function(y, eta, a = 0, b = Inf) {
 dtruncgamma <- dtrunc.trunc_gamma
 
 #' @export
-init.parms.trunc_gamma <- function(y) {
+empiricalParameters.trunc_gamma <- function(y, ...) {
   # Returns  parameter estimates mean and sd
   amean <- mean(y)
   avar <- var(y)
   shp <- amean^2 / avar
   parms <- c(shape = shp, rate = shp / amean)
-  class(parms) <- "trunc_gamma"
+  class(parms) <- "parms_gamma"
   return(parms)
 }
 
@@ -50,24 +47,25 @@ sufficientT.trunc_gamma <- function(y) {
   return(suff.T = cbind(log(y), y))
 }
 
-averageT.trunc_gamma <- function(y) {
-  return(apply(cbind(log(y), y), 2, mean))
-}
-
 #' @export
-natural2parameters.trunc_gamma <- function(eta) {
+natural2parameters.parms_gamma <- function(eta, ...) {
   # eta: The natural parameters in a gamma distribution
   # returns (shape,rate)
-  parms <- c(shape = eta[1] + 1, rate = -eta[2])
+  if (length(eta) != 2) stop("Eta must be a vector of two elements")
+  parms <- c("shape" = eta[[1]] + 1, "rate" = -eta[[2]])
   class(parms) <- class(eta)
   return(parms)
 }
 
 #' @export
-parameters2natural.trunc_gamma <- function(parms) {
+parameters2natural.parms_gamma <- function(parms, ...) {
   # parms: The parameters shape and rate in a gamma distribution
   # returns the natural parameters
-  eta <- c(eta.1 = parms[1] - 1, eta.2 = -parms[2])
+  if (all(c("shape", "rate") %in% names(parms))) {
+    eta <- c(eta1 = parms[["shape"]] - 1, eta2 = -parms[["rate"]])
+  } else {
+    eta <- c(eta1 = parms[["shape"]] - 1, eta2 = -1 / parms[["scale"]])
+  }
   class(eta) <- class(parms)
   return(eta)
 }
@@ -79,12 +77,24 @@ getYseq.trunc_gamma <- function(y, y.min = 1e-6, y.max, n = 100) {
   lo <- max(y.min, mean - 5 * sd)
   hi <- min(y.max, mean + 5 * sd)
   out <- seq(lo, hi, length = n)
+  out <- out[out > 0] # prevents NaN as sufficient statistics
   class(out) <- class(y)
   return(out)
 }
 
-getGradETinv.trunc_gamma <- function(eta, ...) {
+getGradETinv.parms_gamma <- function(eta, ...) {
   # eta: Natural parameter
   # return the inverse of E.T differentiated with respect to eta' : p x p matrix
-  return(A = solve(matrix(c(-1 / eta[1]^2 + dpsi.dx(eta[1]), -1 / eta[2], -1 / eta[2], (eta[1] + 1) / eta[2]^2), ncol = 2)))
+  dpsi.dx <- function(x, k = 10000) {
+    # Returns the derivative of the psi function (removed)
+    sum((1 / ((0:k) + x))^2)
+  }
+  A_inv <- matrix(
+    c(
+      -1 / eta[1]^2 + dpsi.dx(eta[1]), -1 / eta[2],
+      -1 / eta[2], (eta[1] + 1) / eta[2]^2
+    ),
+    ncol = 2
+  )
+  return(A = solve(A_inv))
 }
